@@ -1537,7 +1537,11 @@ class WorkflowBase(Base):
 
     def get_relation_data(self, work):
         ret = {'work': {'workload_id': work.workload_id,
-                        'external_id': work.external_id}}
+                        'external_id': work.external_id,
+                        'work_name': work.get_work_name()}}
+        if hasattr(work, 'get_ancestry_works'):
+            ret['work']['ancestry_works'] = work.get_ancestry_works()
+
         next_works = work.next_works
         if next_works:
             next_works_data = []
@@ -1551,12 +1555,64 @@ class WorkflowBase(Base):
             ret['next_works'] = next_works_data
         return ret
 
+    def organzie_based_on_ancestry_works(self, works):
+        new_ret = []
+
+        ordered_items = {}
+        left_items = []
+        for item in works:
+            if type(item) in [dict]:
+                if 'ancestry_works' not in item['work'] or not item['work']['ancestry_works']:
+                    new_ret.append(item)
+                    ordered_items[item['work']['work_name']] = item
+                else:
+                    # ancestry_works = item['work']['ancestry_works']
+                    left_items.append(item)
+            elif type(item) in [list]:
+                # subworkflow
+                # work_names, ancestry_works = self.get_workflow_ancestry_works(item)
+                # if not ancestry_works:
+                #     new_ret.append(item)
+                # currently now support to use dependency_map to depend_on a workflow.
+                # depending on a workflow should use Condition. It's already processed.
+                new_ret.append(item)
+        while True:
+            new_left_items = left_items
+            left_items = []
+            has_updates = False
+            for item in new_left_items:
+                ancestry_works = item['work']['ancestry_works']
+                all_ancestry_ready = True
+                for work_name in ancestry_works:
+                    if work_name not in ordered_items and work_name != item['work']['work_name']:
+                        all_ancestry_ready = False
+                if all_ancestry_ready:
+                    for work_name in ancestry_works:
+                        if work_name != item['work']['work_name']:
+                            if 'next_works' not in ordered_items[work_name]:
+                                ordered_items[work_name]['next_works'] = [item]
+                            else:
+                                ordered_items[work_name]['next_works'].append(item)
+                            has_updates = True
+                            ordered_items[item['work']['work_name']] = item
+                else:
+                    left_items.append(item)
+            if not has_updates or not left_items:
+                break
+        for item in left_items:
+            new_ret.append(item)
+        return new_ret
+
     def get_relation_map(self):
         ret = []
         init_works = self.init_works
         for internal_id in init_works:
-            work_data = self.get_relation_data(self.works[internal_id])
+            if isinstance(self.works[internal_id], Workflow):
+                work_data = self.works[internal_id].get_relation_map()
+            else:
+                work_data = self.get_relation_data(self.works[internal_id])
             ret.append(work_data)
+        ret = self.organzie_based_on_ancestry_works(ret)
         return ret
 
     def clean_works(self):
